/*
   Copyright (C) 2003-2007  Mathias Broxvall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

%{
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>  
#include "lex.yy.c"

  extern int npoints,nextPoint,burn_steps;
  extern double gravity, repell, friction, dist_off_r, dist_off_g, dist_off_f;
  extern double radiation, transfer, friction_heat;
  extern double fusion_radiation, fusion_chance, fusion_heat;
  extern double m_res;
  extern int m_complicated, m_width, m_height;

  extern void sphere(int,int,double,double,double);
  extern void cube(int,int,double,double,double);

  extern void spin(int,int,double,double,double,double);
  extern void linear(int,int,double,double);
  extern void drift(int,int,double);
  extern void zero(int,int);

  /* Needed to keep track of recursivly loaded files */
  int nfiles=0,filenames[256];
  YY_BUFFER_STATE filestates[256];

  void yy_switch_to_buffer(YY_BUFFER_STATE);
  YY_BUFFER_STATE yy_create_buffer(FILE*,int);      
  void yy_delete_buffer(YY_BUFFER_STATE);

  void prepareBuffers() {
	filestates[0] = yy_create_buffer(stdin,16384);
	yy_switch_to_buffer(filestates[0]);
	filenames[0] = -1;
	nfiles = 0;
  }
    
  int yyerror(char *str) {
	fprintf(stderr,"Syntax error in %s\n",filenames[nfiles]==-1?"stdin":symboltable[filenames[nfiles]]);
	exit(0);
	return 0;
  }

  int yywrap() { 
	if(nfiles>0) { 
	  yy_switch_to_buffer(filestates[nfiles-1]);
	  yy_delete_buffer(filestates[nfiles]);
	  nfiles--;
	  return 0;
	}
	return 1;
  }

  void currFile() {
	fprintf(stderr,"%s:",filenames[nfiles]==-1?"stdin":symboltable[filenames[nfiles]]);
  }

  int yylex();
%}

%token STRING
%token INTEGER
%token FLOAT
%token DONE
%token PERCENT

%token INCLUDE
%token SEED
%token NPOINTS
%token GRAVITY REPELL FRICTION
%token DIST_OFF_G DIST_OFF_R DIST_OFF_F
%token RADIATION TRANSFER FRICTION_HEAT

%token X Y RAD SPEED

%token SPHERE CUBE
%token SPIN LINEAR DRIFT ZERO
%token FUSION_RADIATION FUSION_CHANCE FUSION_HEAT
%token M_SIZE M_RES M_RAD
%token BURN_STEPS

%%

commands : 
DONE { return 0; }
| command commands
;

command : INCLUDE STRING { 
  FILE *fp = fopen(symboltable[$2],"r");
  if(!fp) {
    currFile();
    fprintf(stderr,"could not open %s\n",symboltable[$2]);
    exit(0);
  }
  nfiles++;
  filenames[nfiles]=$2;
  filestates[nfiles] = yy_create_buffer(fp,16384);
  yy_switch_to_buffer(filestates[nfiles]);
}
| SEED INTEGER { srand($2); printf("seeding %d\n",$2); }
| NPOINTS INTEGER {   
  npoints=$2; 
  cube(0,npoints,0.0,0.0,1.0);
}
| BURN_STEPS INTEGER { burn_steps=$2; }
| GRAVITY FLOAT { gravity*=*((float*)&$2); }
| REPELL FLOAT { repell*=*((float*)&$2); }
| FRICTION FLOAT { friction*=*((float*)&$2); }
| DIST_OFF_G FLOAT { dist_off_g*=*((float*)&$2); }
| DIST_OFF_R FLOAT { dist_off_r*=*((float*)&$2); }
| DIST_OFF_F FLOAT { dist_off_f*=*((float*)&$2); }
| RADIATION INTEGER PERCENT { radiation = $2 / 100.0; }
| M_SIZE INTEGER {
  int s=$2&(~63);
  if(s < 0 || s >= 1024) { fprintf(stderr,"bad m_size\n"); exit(-1); }
  m_width = m_height = s;
}
| M_RAD INTEGER {
  if($2 < 0 || $2 > 128) { fprintf(stderr,"bad m_rad\n"); exit(-1); }
  m_complicated = $2;
}
| M_RES FLOAT {
  float f = *((float*)&$2);
  if(f < 0.0 || f > 1024.0) { fprintf(stderr,"bad m_res\n"); exit(-1); }
  m_res = f;
}
| TRANSFER FLOAT { transfer*=*((float*)&$2); }
| FRICTION_HEAT FLOAT { friction_heat*=*((float*)&$2); }
| FUSION_RADIATION FLOAT { fusion_radiation*=*((float*)&$2); }
| FUSION_HEAT FLOAT { fusion_heat*=*((float*)&$2); }
| FUSION_CHANCE FLOAT { fusion_chance*=*((float*)&$2); }
| SPHERE INTEGER PERCENT INTEGER PERCENT X FLOAT Y FLOAT RAD FLOAT {
  sphere(($2*npoints)/100,($4*npoints)/100,*((float*)&$7),*((float*)&$9),*((float*)&$11));
}
| CUBE INTEGER PERCENT INTEGER PERCENT X FLOAT Y FLOAT RAD FLOAT {
  cube(($2*npoints)/100,($4*npoints)/100,*((float*)&$7),*((float*)&$9),*((float*)&$11));
}
| SPIN INTEGER PERCENT INTEGER PERCENT X FLOAT Y FLOAT RAD FLOAT SPEED FLOAT {
  spin(($2*npoints)/100,($4*npoints)/100,*((float*)&$7),*((float*)&$9),*((float*)&$11),*((float*)&$13));
}
| LINEAR INTEGER PERCENT INTEGER PERCENT X FLOAT Y FLOAT  {
  linear(($2*npoints)/100,($4*npoints)/100,*((float*)&$7),*((float*)&$9));
}
| DRIFT INTEGER PERCENT INTEGER PERCENT SPEED FLOAT {
  drift(($2*npoints)/100,($4*npoints)/100,*((float*)&$7));
}
| ZERO INTEGER PERCENT INTEGER PERCENT {
  zero(($2*npoints)/100,($4*npoints)/100);
}

;
%%


