%{
  int install_id(char *,int);
  extern int silent;
  extern int nfiles;
  extern int yylval;
%}

delimiter	[ \t]
ws		{delimiter}+
uletter		[A-Z]
lletter         [a-z]
letter          [A-Za-z]
digit   		[0-9]
integer         {digit}*
pos_float       {digit}*"."{digit}*
neg_float       "-"{pos_float}
string          \"[^\"]*(\\\"[^\"]*)*\"
%%

[\n]            {if(nfiles==0&&!silent)printf("> ");fflush(stdout);}
{ws}		{/* Do nothing */}
\/\/[^\n]*      {/* Do nothing */}
\/\*[^*]*\*\/   {/* Do nothing */}

"%"             {return PERCENT;}
"include"       {return INCLUDE;}
"seed:"         {return SEED;}
"npoints:"      {return NPOINTS;}
"gravity:"      {return GRAVITY;}
"repell:"       {return REPELL;}
"friction:"     {return FRICTION;}
"radiation:"    {return RADIATION;}
"transfer:"     {return TRANSFER;}
"friction_heat:" {return FRICTION_HEAT;}
"dist_off_r:"   {return DIST_OFF_R;}
"dist_off_g:"   {return DIST_OFF_G;}
"dist_off_f:"   {return DIST_OFF_F;}
"sphere"        {return SPHERE;}
"cube"          {return CUBE;}
"spin"          {return SPIN;}
"linear"        {return LINEAR;}
"drift"         {return DRIFT;}
"x:"            {return X;}
"y:"            {return Y;}
"radius:"       {return RAD;}
"speed:"        {return SPEED;}
"fusion_radiation:"  {return FUSION_RADIATION;}
"fusion_heat:"  {return FUSION_HEAT;}
"fusion_chance:" {return FUSION_CHANCE;}
"m_size:"       {return M_SIZE;}
"m_rad:"        {return M_RAD;}
"m_res:"        {return M_RES;}
"burn_steps:"   {return BURN_STEPS;}
{string}        {yylval=install_id(yytext+1,yyleng-2); return STRING;}
{integer}       {yylval=atoi(yytext); return INTEGER;}
{pos_float}     {*((float*)&yylval)=(float)atof(yytext); return FLOAT;}
{neg_float}     {*((float*)&yylval)=(float)atof(yytext); return FLOAT;}
<<EOF>>         {return DONE;}
%%

#include <strings.h>

#define MAX_IDENTIFIERS 10000
char *symboltable[MAX_IDENTIFIERS];
int number_of_identifiers=0;

int install_id(char *yytext,int yyleng) {
  int i;
  char str[256];

  strncpy(str,yytext,yyleng);
  str[yyleng]=0;

  for(i=0;i<number_of_identifiers;i++)
    if(strcmp(str,symboltable[i])==0) return i;
  if(number_of_identifiers >= MAX_IDENTIFIERS) {
    printf("Tokenizer error.\nMaximum number of identifiers reached.\nChange 'lex.l' and recompile!\n");
    exit(0);
  }
  symboltable[i] = strdup(str);
  number_of_identifiers++;
  return i;
}

